@@ -77,6 +77,10 @@ namespace Banshee.Collection.Database
             this.provider = provider;
             this.source = source;
         }
+        
+        protected HyenaSqliteConnection Connection {
+            get { return connection; }
+        }
 
         private bool initialized = false;
         public void Initialize (IDatabaseTrackModelCache cache)
@@ -124,8 +128,10 @@ namespace Banshee.Collection.Database
 
         protected virtual void GenerateSortQueryPart ()
         {
-            SortQuery = (SortColumn == null)
-                ? BansheeQuery.GetSort ("Artist", true)
+            SortQuery = (SortColumn == null || SortColumn.SortType == SortType.None)
+                ? (SortColumn != null && source is Banshee.Playlist.PlaylistSource)
+                    ? "CorePlaylistEntries.ViewOrder ASC, CorePlaylistEntries.EntryID ASC" 
+                    : BansheeQuery.GetSort ("Artist", true)
                 : BansheeQuery.GetSort (SortColumn.SortKey, SortColumn.SortType == SortType.Ascending);
         }
 
@@ -137,11 +143,18 @@ namespace Banshee.Collection.Database
                 }
                 
                 if (sort_column == column && sort_column != null) {
-                    sort_column.SortType = sort_column.SortType == SortType.Ascending 
-                        ? SortType.Descending 
-                        : SortType.Ascending;
+                    switch (sort_column.SortType) {
+                        case SortType.Ascending:    sort_column.SortType = SortType.Descending; break;
+                        case SortType.Descending:   sort_column.SortType = SortType.None; break;
+                        case SortType.None:         sort_column.SortType = SortType.Ascending; break;
+                    }
                 }
-            
+                
+                // If we're switching from a different column, or we aren't in a playlist, make sure sort type isn't None
+                if (sort_column != null && (sort_column != column || !(source is Banshee.Playlist.PlaylistSource)) && sort_column.SortType == SortType.None) {
+                    sort_column.SortType = SortType.Ascending;
+                }
+                
                 sort_column = column;
             
                 GenerateSortQueryPart ();
@@ -165,8 +178,12 @@ namespace Banshee.Collection.Database
 
         public void InvalidateCache ()
         {
-            cache.ClearManagedCache ();
-            OnReloaded ();
+            if (cache == null) {
+                Log.ErrorFormat ("Called invalidate cache for {0}'s track model, but cache is null", source);
+            } else {
+                cache.Clear ();
+                OnReloaded ();
+            }
         }
 
         private string unfiltered_query;
@@ -174,18 +191,14 @@ namespace Banshee.Collection.Database
             get {
                 return unfiltered_query ?? unfiltered_query = String.Format (
                     "FROM {0}{1} WHERE {2} {3}",
-                    provider.From, JoinFragment, provider.Where, ConditionFragment
+                    provider.From, JoinFragment,
+                    String.IsNullOrEmpty (provider.Where) ? "1=1" : provider.Where,
+                    ConditionFragment
                 );
             }
         }
 
-        /*public void UpdateAggregates ()
-        {
-            UpdateUnfilteredAggregates ();
-            UpdateFilteredAggregates ();
-        }*/
-
-        private void UpdateUnfilteredAggregates ()
+        protected virtual void UpdateUnfilteredAggregates ()
         {
             HyenaSqliteCommand count_command = new HyenaSqliteCommand (String.Format (
                 "SELECT COUNT(*) {0}", UnfilteredQuery
@@ -193,12 +206,6 @@ namespace Banshee.Collection.Database
             count = connection.Query<long> (count_command);
         }
 
-        /*private void UpdateFilteredAggregates ()
-        {
-            cache.UpdateAggregates ();
-            filtered_count = cache.Count;
-        }*/
-        
         public override void Reload ()
         {
             Reload (null);
@@ -206,6 +213,12 @@ namespace Banshee.Collection.Database
 
         public void Reload (IListModel reloadTrigger)
         {
+            if (cache == null) {
+                Log.WarningFormat ("Called Reload on {0} for source {1} but cache is null;  Did you forget to call AfterInitialized () in your DatabaseSource ctor?",
+                    this, source == null ? "null source!" : source.Name);
+                return;
+            }
+
             lock (this) {
                 GenerateUserQueryFragment ();
 
@@ -214,10 +227,10 @@ namespace Banshee.Collection.Database
 
                 List<IFilterListModel> reload_models = new List<IFilterListModel> ();
                 bool found = (reloadTrigger == null);
-                foreach (IFilterListModel model in source.FilterModels) {
+                foreach (IFilterListModel filter in source.CurrentFilters) {
                     if (found) {
-                        reload_models.Add (model);
-                    } else if (model == reloadTrigger) {
+                        reload_models.Add (filter);
+                    } else if (filter == reloadTrigger) {
                         found = true;
                     }
                 }
@@ -232,8 +245,8 @@ namespace Banshee.Collection.Database
                     }
                     
                     bool have_filters = false;
-                    foreach (IFilterListModel model in source.FilterModels) {
-                        have_filters |= !model.Selection.AllSelected;
+                    foreach (IFilterListModel filter in source.CurrentFilters) {
+                        have_filters |= !filter.Selection.AllSelected;
                     }
                     
                     // Unless both artist/album selections are "all" (eg unfiltered), reload
@@ -268,17 +281,17 @@ namespace Banshee.Collection.Database
             qb.Append (UnfilteredQuery);
             
             if (with_filters) {
-                foreach (IFilterListModel model in source.FilterModels) {
-                    string filter = GetFilterFromModel (model);
-                    if (filter != null) {
-                        qb.Append ("AND");
-                        qb.Append (filter);
+                foreach (IFilterListModel filter in source.CurrentFilters) {
+                    string filter_sql = filter.GetSqlFilter ();
+                    if (filter_sql != null) {
+                        qb.Append (" AND ");
+                        qb.Append (filter_sql);
                     }
                 }
             }
             
             if (query_fragment != null) {
-                qb.Append ("AND ");
+                qb.Append (" AND ");
                 qb.Append (query_fragment);
             }
             
@@ -293,8 +306,7 @@ namespace Banshee.Collection.Database
 
         public override int IndexOf (TrackInfo track)
         {
-            DatabaseTrackInfo db_track = track as DatabaseTrackInfo;
-            return (int) (db_track == null ? -1 : cache.IndexOf ((int)db_track.TrackId));
+            return (int) cache.IndexOf (track as DatabaseTrackInfo);
         }
 
         private DateTime random_began_at = DateTime.MinValue;
@@ -343,6 +355,7 @@ namespace Banshee.Collection.Database
         
         public int UnfilteredCount {
             get { return (int) count; }
+            set { count = value; }
         }
 
         public string UserQuery {
@@ -402,80 +415,10 @@ namespace Banshee.Collection.Database
         private string PrefixCondition (string prefix)
         {
             string condition = Condition;
-            if (condition == null || condition == String.Empty)
-                return String.Empty;
-            else
-                return String.Format (" {0} {1} ", prefix, condition);
+            return String.IsNullOrEmpty (condition)
+                ? String.Empty
+                : String.Format (" {0} {1} ", prefix, condition);
         }
-        
-        private string GetFilterFromModel (IFilterListModel model)
-        {
-            string filter = null;
-            
-            ModelHelper.BuildIdFilter<object> (model.GetSelectedObjects (), model.FilterColumn, null,
-                delegate (object item) { return model.ItemToFilterValue (item); },
-                delegate (string new_filter) { filter = new_filter; }
-            );
-            
-            return filter;
-        }
-
-        /*public override IEnumerable<ArtistInfo> ArtistInfoFilter {
-            set {
-                ModelHelper.BuildIdFilter<ArtistInfo> (value, "CoreTracks.ArtistID", artist_id_filter_query,
-                    delegate (ArtistInfo artist) {
-                        if (!(artist is DatabaseArtistInfo)) {
-                            return null;
-                        }
-                        
-                        return ((DatabaseArtistInfo)artist).DbId.ToString ();
-                    },
-                
-                    delegate (string new_filter) {
-                        artist_id_filter_query = new_filter;
-                        Console.WriteLine ("artist filter now set to {0}", artist_id_filter_query);
-                    }
-                );
-            }
-        }
-        
-        public override IEnumerable<AlbumInfo> AlbumInfoFilter {
-            set { 
-                ModelHelper.BuildIdFilter<AlbumInfo> (value, "CoreTracks.AlbumID", album_id_filter_query,
-                    delegate (AlbumInfo album) {
-                        if (!(album is DatabaseAlbumInfo)) {
-                            return null;
-                        }
-                        
-                        return ((DatabaseAlbumInfo)album).DbId.ToString ();
-                    },
-                
-                    delegate (string new_filter) {
-                        album_id_filter_query = new_filter;
-                        Console.WriteLine ("album filter now set to {0}", album_id_filter_query);
-                    }
-                );
-            }
-        }*/
-
-        public override void ClearArtistAlbumFilters ()
-        {
-            Reload ();
-        }
-
-        /*private HyenaSqliteCommand check_artists_command = new HyenaSqliteCommand (
-            "SELECT ItemID FROM CoreCache WHERE ModelID = ? AND ItemID NOT IN (SELECT ArtistID FROM CoreArtists)"
-        );
-
-        private HyenaSqliteCommand check_albums_command = new HyenaSqliteCommand (
-            "SELECT ItemID FROM CoreCache WHERE ModelID = ? AND ItemID NOT IN (SELECT AlbumID FROM CoreAlbums)"
-        );*/
-
-        /*public void CheckFilters ()
-        {
-            if (track_model.Artist
-            if (ServiceManager.DbConnection.Query<int> (
-        }*/
 
         public int CacheId {
             get { return (int) cache.CacheId; }
@@ -536,5 +479,7 @@ namespace Banshee.Collection.Database
         public string ReloadFragment {
             get { return reload_fragment; }
         }
+        
+        public bool CachesValues { get { return false; } }
     }
 }
