@@ -47,7 +47,6 @@ namespace Banshee.Gui
 {
     public class TrackActions : BansheeActionGroup
     {
-        private InterfaceActionService action_service;
         private RatingActionProxy rating_proxy;
 
         private static readonly string [] require_selection_actions = new string [] {
@@ -56,29 +55,10 @@ namespace Banshee.Gui
             "RateTracksAction", "SelectNoneAction"
         };
         
-        private IHasTrackSelection track_selector;
-        public IHasTrackSelection TrackSelector {
-            get { return track_selector; }
-            set {
-                if (track_selector != null && track_selector != value) {
-                    track_selector.TrackSelectionProxy.Changed -= HandleSelectionChanged;
-                    track_selector.TrackSelectionProxy.SelectionChanged -= HandleSelectionChanged;
-                }
+        public event EventHandler SelectionChanged;
 
-                track_selector = value;
-
-                if (track_selector != null) {
-                    track_selector.TrackSelectionProxy.Changed += HandleSelectionChanged;
-                    track_selector.TrackSelectionProxy.SelectionChanged += HandleSelectionChanged;
-                    UpdateActions ();
-                }
-            }
-        }
-
-        public TrackActions (InterfaceActionService actionService) : base ("Track")
+        public TrackActions (InterfaceActionService actionService) : base (actionService, "Track")
         {
-            action_service = actionService;
-
             Add (new ActionEntry [] {
                 new ActionEntry("TrackContextMenuAction", null, 
                     String.Empty, null, null, OnTrackContextMenu),
@@ -106,12 +86,12 @@ namespace Banshee.Gui
                     OnAddToNewPlaylist),
 
                 new ActionEntry ("RemoveTracksAction", Stock.Remove,
-                    Catalog.GetString("_Remove"), "Delete",
-                    Catalog.GetString("Remove selected track(s) from this source"), OnRemoveTracks),
+                    Catalog.GetString ("_Remove"), "Delete",
+                    Catalog.GetString ("Remove selected track(s) from this source"), OnRemoveTracks),
 
                 new ActionEntry ("RemoveTracksFromLibraryAction", null,
-                    Catalog.GetString("Remove From _Library"), "",
-                    Catalog.GetString("Remove selected track(s) from library"), OnRemoveTracksFromLibrary),
+                    Catalog.GetString ("Remove From _Library"), "",
+                    Catalog.GetString ("Remove selected track(s) from library"), OnRemoveTracksFromLibrary),
 
                 new ActionEntry ("DeleteTracksFromDriveAction", null,
                     Catalog.GetString ("_Delete From Drive"), null,
@@ -131,15 +111,11 @@ namespace Banshee.Gui
                 new ActionEntry ("SearchForSameArtistAction", null,
                     Catalog.GetString ("By Matching A_rtist"), null,
                     Catalog.GetString ("Search all songs of this artist"), OnSearchForSameArtist),
-
-                //new ActionEntry ("JumpToPlayingTrackAction", null,
-                //    Catalog.GetString ("_Jump to playing song"), "<control>J",
-                //    null, OnJumpToPlayingTrack),
             });
 
-            action_service.UIManager.ActionsChanged += HandleActionsChanged;
+            Actions.UIManager.ActionsChanged += HandleActionsChanged;
 
-            action_service.GlobalActions["EditMenuAction"].Activated += HandleEditMenuActivated;
+            Actions.GlobalActions["EditMenuAction"].Activated += HandleEditMenuActivated;
             ServiceManager.SourceManager.ActiveSourceChanged += HandleActiveSourceChanged;
 
             this["AddToPlaylistAction"].HideIfEmpty = false;
@@ -147,23 +123,36 @@ namespace Banshee.Gui
 
 #region State Event Handlers
 
+        private ITrackModelSource current_source;
         private void HandleActiveSourceChanged (SourceEventArgs args)
         {
+            if (current_source != null && current_source.TrackModel != null) {
+                current_source.TrackModel.Selection.Changed -= HandleSelectionChanged;
+                current_source = null;
+            }
+            
+            ITrackModelSource new_source = ActiveSource as ITrackModelSource;
+            if (new_source != null) {
+                new_source.TrackModel.Selection.Changed += HandleSelectionChanged;
+                current_source = new_source;
+            }
+            
             UpdateActions ();
         }
 
         private void HandleActionsChanged (object sender, EventArgs args)
         {
-            if (action_service.UIManager.GetAction ("/MainMenu/EditMenu") != null) {
-                rating_proxy = new RatingActionProxy (action_service.UIManager, this["RateTracksAction"]);
+            if (Actions.UIManager.GetAction ("/MainMenu/EditMenu") != null) {
+                rating_proxy = new RatingActionProxy (Actions.UIManager, this["RateTracksAction"]);
                 rating_proxy.AddPath ("/MainMenu/EditMenu", "AddToPlaylist");
                 rating_proxy.AddPath ("/TrackContextMenu", "AddToPlaylist");
-                action_service.UIManager.ActionsChanged -= HandleActionsChanged;
+                Actions.UIManager.ActionsChanged -= HandleActionsChanged;
             }
         }
 
         private void HandleSelectionChanged (object sender, EventArgs args)
         {
+            OnSelectionChanged ();
             UpdateActions ();
         }
 
@@ -171,6 +160,14 @@ namespace Banshee.Gui
         {
             ResetRating ();
         }
+        
+        private void OnSelectionChanged ()
+        {
+            EventHandler handler = SelectionChanged;
+            if (handler != null) {
+                handler (this, EventArgs.Empty);
+            }
+        }
 
 #endregion
 
@@ -197,18 +194,18 @@ namespace Banshee.Gui
 
         private void UpdateActions ()
         {
-            if (TrackSelector == null || TrackSelector.TrackSelectionProxy == null) {
-                return;
-            }
-            
-            Hyena.Collections.Selection selection = TrackSelector.TrackSelectionProxy.Selection;
             Source source = ServiceManager.SourceManager.ActiveSource;
             bool in_database = source is DatabaseSource;
+            PrimarySource primary_source = (source as PrimarySource) ?? (source.Parent as PrimarySource);
+            
+            Hyena.Collections.Selection selection = (source is ITrackModelSource) ? (source as ITrackModelSource).TrackModel.Selection : null;
 
             if (selection != null) {
+                Sensitive = Visible = true;
                 bool has_selection = selection.Count > 0;
-                foreach (string action in require_selection_actions)
+                foreach (string action in require_selection_actions) {
                     this[action].Sensitive = has_selection;
+                }
 
                 bool has_single_selection = selection.Count == 1;
 
@@ -226,24 +223,28 @@ namespace Banshee.Gui
                     UpdateAction ("DeleteTracksFromDriveAction", is_track_source && track_source.CanDeleteTracks, has_selection, source);
                     UpdateAction ("RemoveTracksFromLibraryAction", source.Parent is LibrarySource, has_selection, null);
                     
-                    UpdateAction ("TrackPropertiesAction", in_database, has_selection, null);
+                    UpdateAction ("TrackPropertiesAction", in_database, has_selection, source);
                     UpdateAction ("RateTracksAction", in_database, has_selection, null);
-                    UpdateAction ("AddToPlaylistAction", in_database, has_selection, null);
+                    UpdateAction ("AddToPlaylistAction", in_database && primary_source != null && primary_source.SupportsPlaylists, has_selection, null);
                 }
+            } else {
+                Sensitive = Visible = false;
             }
         }
 
         private void ResetRating ()
         {
-            int rating = 0;
-
-            // If there is only one track, get the preset rating
-            if (TrackSelector.TrackSelectionProxy.Selection.Count == 1) {
-                foreach (TrackInfo track in TrackSelector.GetSelectedTracks ()) {
-                    rating = track.Rating;
+            if (current_source != null) {
+                int rating = 0;
+    
+                // If there is only one track, get the preset rating
+                if (current_source.TrackModel.Selection.Count == 1) {
+                    foreach (TrackInfo track in current_source.TrackModel.SelectedItems) {
+                        rating = track.Rating;
+                    }
                 }
+                rating_proxy.Reset (rating);
             }
-            rating_proxy.Reset (rating);
         }
 
 #endregion
@@ -252,42 +253,36 @@ namespace Banshee.Gui
 
         private void OnSelectAll (object o, EventArgs args)
         {
-            TrackSelector.TrackSelectionProxy.Selection.SelectAll ();
+            if (current_source != null)
+                current_source.TrackModel.Selection.SelectAll ();
         }
 
         private void OnSelectNone (object o, EventArgs args)
         {
-            TrackSelector.TrackSelectionProxy.Selection.Clear ();
+            if (current_source != null)
+                current_source.TrackModel.Selection.Clear ();
         }
 
         private void OnTrackContextMenu (object o, EventArgs args)
         {
             ResetRating ();
-
-            Gtk.Menu menu = action_service.UIManager.GetWidget ("/TrackContextMenu") as Menu;
-            if (menu == null || menu.Children.Length == 0) {
-                return;
-            }
-
-            int visible_children = 0;
-            foreach (Widget child in menu)
-                if (child.Visible)
-                    visible_children++;
-
-            if (visible_children == 0) {
-                return;
-            }
-
-            menu.Show (); 
-            menu.Popup (null, null, null, 0, Gtk.Global.CurrentEventTime);
+            ShowContextMenu ("/TrackContextMenu");
         }
 
         private void OnTrackProperties (object o, EventArgs args)
         {
-            TrackEditor propEdit = new TrackEditor (TrackSelector.GetSelectedTracks ());
-            propEdit.Saved += delegate {
-                //ui.playlistView.QueueDraw();
-            };
+            if (current_source != null) {
+                Source source = current_source as Source;
+                InvokeHandler handler = source != null 
+                    ? source.GetInheritedProperty<InvokeHandler> ("TrackPropertiesActionHandler") 
+                    : null;
+                
+                if (handler != null) {
+                    handler ();
+                } else {
+                    new TrackEditor (current_source.TrackModel.SelectedItems);
+                }
+            }
         }
 
         // Called when the Add to Playlist action is highlighted.
@@ -392,14 +387,6 @@ namespace Banshee.Gui
 
         private void OnRateTracks (object o, EventArgs args)
         {
-            foreach (Widget proxy in (o as Gtk.Action).Proxies) {
-                Menu menu = proxy.Parent as Menu;
-                if (menu != null && menu.Visible) {
-                    menu.Popdown ();
-                    menu.Hide ();
-                }
-            }
-
             ThreadAssist.SpawnFromMain (delegate {
                 (ActiveSource as DatabaseSource).RateSelectedTracks (rating_proxy.LastRating);
             });
@@ -407,26 +394,24 @@ namespace Banshee.Gui
 
         private void OnSearchForSameArtist (object o, EventArgs args)
         {
-            Source source = ActiveSource;
-            foreach (TrackInfo track in TrackSelector.GetSelectedTracks ()) {
-                source.FilterQuery = BansheeQuery.ArtistField.ToTermString (":", track.ArtistName);
-                break;
+            if (current_source != null) {
+                foreach (TrackInfo track in current_source.TrackModel.SelectedItems) {
+                    ActiveSource.FilterQuery = BansheeQuery.ArtistField.ToTermString (":", track.ArtistName);
+                    break;
+                }
             }
         }
 
         private void OnSearchForSameAlbum (object o, EventArgs args)
         {
-            Source source = ActiveSource;
-            foreach (TrackInfo track in TrackSelector.GetSelectedTracks ()) {
-                source.FilterQuery = BansheeQuery.AlbumField.ToTermString (":", track.AlbumTitle);
-                break;
+            if (current_source != null) {
+                foreach (TrackInfo track in current_source.TrackModel.SelectedItems) {
+                    ActiveSource.FilterQuery = BansheeQuery.AlbumField.ToTermString (":", track.AlbumTitle);
+                    break;
+                }
             }
         }
 
-        /*private void OnJumpToPlayingTrack (object o, EventArgs args)
-        {
-        }*/
-
 #endregion
 
         private static bool ConfirmRemove (ITrackModelSource source, bool delete, int selCount)
