@@ -33,7 +33,7 @@ using Gtk;
 
 namespace Hyena.Data.Gui
 {
-    public partial class ListView<T> : Widget
+    public partial class ListView<T> : ListViewBase
     {
         internal struct CachedColumn
         {
@@ -91,6 +91,7 @@ namespace Hyena.Data.Gui
                     continue;
                 }
                 
+                // If we don't already have a MinWidth set, use the width of our Title text
                 if (column.MinWidth == 0) {
                     int w;
                     int h;
@@ -129,6 +130,7 @@ namespace Hyena.Data.Gui
             }
             
             for (int i = 0; i < column_cache.Length; i++) {
+                // Calculate this column's proportional share of the width, and set positions (X1/X2)
                 column_cache[i].Width = (int)Math.Round (((double)header_width * column_cache[i].Column.Width));
                 column_cache[i].X1 = i == 0 ? 0 : column_cache[i - 1].X2;
                 column_cache[i].X2 = column_cache[i].X1 + column_cache[i].Width;
@@ -170,9 +172,8 @@ namespace Hyena.Data.Gui
                 if (header_cell != null) {
                     min_width = header_cell.MinWidth;
                 }
-                column_cache[i].MinWidth = min_width;
-                column_cache[i].MaxWidth = column_cache[i].Column.MaxWidth;
                 min_header_width += column_cache[i].MinWidth = Math.Max (min_width, column_cache[i].Column.MinWidth);
+                column_cache[i].MaxWidth = Math.Max (column_cache[i].Column.MaxWidth, column_cache[i].MinWidth);
             }
             
             if (min_header_width >= header_interaction_alloc.Width) {
@@ -236,15 +237,13 @@ namespace Hyena.Data.Gui
                 column_cache[i].ElasticWidth += delta;
             }
             
-            remaining_width = Math.Round (remaining_width);
-            return remaining_width;
+            return Math.Round (remaining_width);
         }
         
         protected virtual void OnColumnControllerUpdated ()
         {
             InvalidateColumnCache ();
             RegenerateColumnCache ();
-            RegenerateCanvases ();
             UpdateAdjustments ();
             QueueDraw ();
         }
@@ -303,7 +302,7 @@ namespace Hyena.Data.Gui
             for (int i = resizing_column_index + 1; i < column_cache.Length; i++) {
                 total_elastic_width += column_cache[i].ElasticWidth = sign == 1
                     ? column_cache[i].Width - column_cache[i].MinWidth
-                    : column_cache[i].Column.MaxWidth - column_cache[i].Width;
+                    : column_cache[i].MaxWidth - column_cache[i].Width;
             }
             
             if (total_elastic_width == 0) {
@@ -332,11 +331,13 @@ namespace Hyena.Data.Gui
                 return null;
             }
             
-            x += (int)hadjustment.Value;
+            x += HadjustmentValue;
             
             for (int i = 0; i < column_cache.Length - 1; i++) {
-                if (x >= column_cache[i].ResizeX1 - 2 && 
-                    x <= column_cache[i].ResizeX2 + 2 &&
+                if (x < column_cache[i].ResizeX1 - 2) {
+                    // No point in checking other columns since their ResizeX1 are even larger
+                    break;
+                } else if (x <= column_cache[i].ResizeX2 + 2 &&
                     column_cache[i].Column.MaxWidth != column_cache[i].Column.MinWidth) {
                     return column_cache[i].Column;
                 }
@@ -351,13 +352,14 @@ namespace Hyena.Data.Gui
                 return null;
             }
             
-            x += (int)hadjustment.Value;
+            x += HadjustmentValue;
             
             foreach (CachedColumn column in column_cache) {
                 if (x >= column.X1 && x <= column.X2) {
                     return column.Column;
                 }
             }
+            
             return null;
         }
         
@@ -375,15 +377,18 @@ namespace Hyena.Data.Gui
         private ColumnController column_controller;
         public ColumnController ColumnController {
             get { return column_controller; }
-            set { 
+            set {
+                if (column_controller == value) {
+                    return;
+                }
+
                 if (column_controller != null) {
                     column_controller.Updated -= OnColumnControllerUpdatedHandler;
                 }
                 
                 column_controller = value;
                 
-                RegenerateColumnCache ();
-                QueueDraw ();
+                OnColumnControllerUpdated ();
                 
                 if (column_controller != null) {
                     column_controller.Updated += OnColumnControllerUpdatedHandler;
@@ -421,7 +426,6 @@ namespace Hyena.Data.Gui
             set { 
                 header_visible = value;
                 MoveResize (Allocation);
-                RegenerateCanvases ();
             }
         }
         
@@ -434,7 +438,7 @@ namespace Hyena.Data.Gui
             private Column column;
             private bool ready = false;
             
-            public ColumnToggleMenuItem (Column column) : base (column.Title)
+            public ColumnToggleMenuItem (Column column) : base (column.Title ?? String.Empty)
             {
                 this.column = column;
                 Active = column.Visible; 
